package com.arms.api.admin.service;


import com.arms.api.admin.constrant.ServiceName;
import com.arms.api.fluentd.entity.FluentdEntity;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

@Service
@AllArgsConstructor
@Slf4j
public class FluentdIndexAdminService extends EsCommonRepositoryWrapper<FluentdEntity> implements IndexAdminService {

    @Override
    public void delete() {
        Set<String> fluentd = this.findIndexNamesByAlias("fluentd");
        List<String> days = dayList(90);
        fluentd.stream()
            .filter(indexName->!days.contains(indexName))
            .collect(Collectors.toList())
            .forEach(indexName-> {
                log.info("delete-index->{}",indexName);
                this.deleteIndex(indexName);
            }
        );
    }

    private List<String> dayList(Integer range) {

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());

        calendar.add(Calendar.DATE, -(Math.abs(range) - 1));
        Date twoDaysAgo = calendar.getTime();

        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
        calendar.setTime(twoDaysAgo);

        return IntStream.range(0, range).boxed().map(a -> {
            String day = formatter.format(calendar.getTime());
            calendar.add(Calendar.DATE, 1);
            return "fluentd-" + day;
        }).collect(Collectors.toList());

    }

    @Override
    public ServiceName serviceName() {
        return ServiceName.FLUENTD;
    }
}
