package com.arms.api.analysis.scope.service;


import com.arms.api.analysis.scope.dto.ScopeAggrDTO;
import com.arms.api.analysis.scope.dto.ScopeDTO;
import com.arms.api.analysis.scope.dto.ScopeParentKeySearchDTO;
import com.arms.api.analysis.scope.vo.ReqStatusesByVersionWithAssigneeVO;
import com.arms.api.analysis.scope.vo.TreeBarTop10IssueVO;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.util.model.dto.PdServiceAndIsReqDTO;
import com.arms.egovframework.javaservice.esframework.esquery.EsBoolQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.model.dto.집계_하위_필드;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentBucket;

import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import static java.util.stream.Collectors.toList;

@Slf4j
@Service("analysisScope")
public class AnalysisScopeImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements AnalysisScope {

    @Override
    public List<ReqStatusesByVersionWithAssigneeVO> reqStatusesByVersionWithAssignees(ScopeDTO scopeDto) {

        List<AlmIssueEntity> reqIssuesWithAssignees = reqIssuesWithAssignee(scopeDto);

        List<AlmIssueEntity> subTaskWithAssignees = subTasks(reqIssueKeyWithAssignee(reqIssuesWithAssignees));

        Map<String, List<AlmIssueEntity>> subTaskWithAssigneeMap = assigneesToMap(subTaskWithAssignees);

        return getReqStatusesByVersionWithAssignees(reqIssuesWithAssignees,subTaskWithAssigneeMap);

    }

    private List<ReqStatusesByVersionWithAssigneeVO> getReqStatusesByVersionWithAssignees(
            List<AlmIssueEntity> reqIssueListWithAssignees
        , Map<String, List<AlmIssueEntity>> subTaskWithAssigneeMap
    ){
        List<ReqStatusesByVersionWithAssigneeVO> reqStatusesByVersionWithAssigneeVOS = new ArrayList<>();

        for(AlmIssueEntity almIssueEntity : reqIssueListWithAssignees) {

            Set<String> mailSet = new HashSet<>();

            mailSet.add(almIssueEntity.getAssignee().getEmailAddress());

            mailSet.addAll(getAssigneeEmailAddresses(subTaskWithAssigneeMap, almIssueEntity.getKey()));

            reqStatusesByVersionWithAssigneeVOS.add(
                getReqStatusesByVersionWithAssignee(almIssueEntity,mailSet)
            );

        }
        return reqStatusesByVersionWithAssigneeVOS;
    }

    private Set<String> getAssigneeEmailAddresses(Map<String, List<AlmIssueEntity>> subTaskWithAssigneeMap, String almIssueEntityKey) {

        Set<String> mailSet = new HashSet<>();

        if(subTaskWithAssigneeMap!=null&&subTaskWithAssigneeMap.containsKey(almIssueEntityKey)) {

            List<AlmIssueEntity> subTasks = subTaskWithAssigneeMap.get(almIssueEntityKey);

            mailSet.addAll(getMailSet(subTasks));

            subTaskWithAssigneeMap.remove(almIssueEntityKey); //담당자 수 매핑이 끝난 요구사항은 Map에서 제거
        }

        return mailSet;
    }

    private Set<String> getMailSet(List<AlmIssueEntity> subTasks) {

        Set<String> resultMailSet = new HashSet<>();

        for(AlmIssueEntity subtask : subTasks) {
            String emailAddress = subtask.getAssignee().getEmailAddress();
            resultMailSet.add(emailAddress);
        }

        return resultMailSet;
    }

    private ReqStatusesByVersionWithAssigneeVO getReqStatusesByVersionWithAssignee(AlmIssueEntity almIssueEntity, Set<String> mailSet){
        return ReqStatusesByVersionWithAssigneeVO.builder()
                .pdServiceVersionLinks(almIssueEntity.getPdServiceVersions())
                .issueKey(almIssueEntity.getKey())
                .statusName(almIssueEntity.getStatus().getName())
                .numOfWorkers(mailSet.size()).build();
    }

    private List<AlmIssueEntity> reqIssuesWithAssignee(ScopeDTO scopeDto){
        List<AlmIssueEntity> reqIssues = this.findIssue(scopeDto);
        return reqIssues.stream()
            .filter(almIssueEntity -> almIssueEntity.getAssignee() != null)
            .collect(toList());
    }

    private ScopeParentKeySearchDTO reqIssueKeyWithAssignee (List<AlmIssueEntity> reqIssueListWithAssignees){
        return new ScopeParentKeySearchDTO(
                reqIssueListWithAssignees
                    .stream()
                    .map(AlmIssueEntity::getKey)
                    .collect(Collectors.toList())
        );
    }

    private Map<String, List<AlmIssueEntity>> assigneesToMap(List<AlmIssueEntity> almIssueEntities){
        return almIssueEntities
                .stream()
                .filter(almIssueEntity -> almIssueEntity.getAssignee() != null)
                .collect(Collectors.groupingBy(AlmIssueEntity::getParentReqKey));
    }

    @Override
    public List<TreeBarTop10IssueVO> treeBarTop10(ScopeAggrDTO scopeAggrDTO){
        DocumentAggregations documentAggregations = this.cReqLinkTop10(scopeAggrDTO);
        return getCReqLinkTop10(documentAggregations);
    }

    private DocumentAggregations cReqLinkTop10(ScopeAggrDTO scopeAggrDTO) {

        scopeAggrDTO.set메인_그룹_필드("cReqLink");
        scopeAggrDTO.set메인_그룹_필드명_별칭("cReqLink");
        
        scopeAggrDTO.set집계_하위_필드들(
            List.of(집계_하위_필드.builder()
                 .하위_필드명_별칭("assigneeDisplayName")
                 .하위_필드명("assignee.assignee_displayName.keyword")
                 .build()
            )
        );
        
        EsBoolQuery[] esBoolQueries = Stream.of(
            new TermQueryFilter("pdServiceId", scopeAggrDTO.getPdServiceLink()),
            new TermsQueryFilter("pdServiceVersions", scopeAggrDTO.getPdServiceVersionLinks()),
            new TermQueryFilter("isReq", scopeAggrDTO.getIsReq())
        ).toArray(EsBoolQuery[]::new);
        
        EsQuery esQuery = new EsQueryBuilder().bool(esBoolQueries);
        
        scopeAggrDTO.setEsQuery(esQuery);

        return this.aggregateRecentDocs(scopeAggrDTO);
    }

    private List<TreeBarTop10IssueVO> getCReqLinkTop10(DocumentAggregations documentAggregations) {
        List<DocumentBucket> cReqLinkCountTop10 = documentAggregations.docBuckets()
            .stream()
            .sorted((a,b)->b.getDocFieldNameCount("cReqLink").compareTo(a.getDocFieldNameCount("cReqLink")))
            .limit(10)
            .collect(Collectors.toList());

        return new DocumentAggregations(cReqLinkCountTop10)
            .deepestSubList()
            .stream()
            .map(a -> TreeBarTop10IssueVO.builder()
                .cReqLink(Long.parseLong(a.getDocFieldName("cReqLink")))
                .assigneeDisplayName(a.getDocFieldName("assigneeDisplayName"))
                .assigneeCount(a.getDocFieldNameCount("assigneeDisplayName"))
                .build())
            .filter(a->a.getAssigneeCount()!=0L)
            .collect(Collectors.toList());
    }

    private List<AlmIssueEntity> subTasks(ScopeParentKeySearchDTO scopeParentKeySearchDTO) {

        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                new TermsQueryFilter("parentReqKey", scopeParentKeySearchDTO.getParentIssueKeys())
            );

        scopeParentKeySearchDTO.setEsQuery(esQuery);

        return this.findRecentHits(scopeParentKeySearchDTO).toDocs();
    }

    private List<AlmIssueEntity> findIssue(ScopeDTO scopeDto) {

        PdServiceAndIsReqDTO pdServiceAndIsReq = scopeDto.getPdServiceAndIsReq();

        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                new TermQueryFilter("isReq", pdServiceAndIsReq.getIsReq()),
                new TermQueryFilter("pdServiceId", pdServiceAndIsReq.getPdServiceId()),
                new TermsQueryFilter("pdServiceVersions", pdServiceAndIsReq.getPdServiceVersions())
            );

        scopeDto.setEsQuery(esQuery);

        return this.findRecentHits(scopeDto).toDocs();
    }

    @Override
    public List<AlmIssueEntity> pdServiceVersions(ScopeDTO scopeDto) {

        PdServiceAndIsReqDTO pdServiceAndIsReq = scopeDto.getPdServiceAndIsReq();

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("isReq", pdServiceAndIsReq.getIsReq()),
                        new TermsQueryFilter("pdServiceId", pdServiceAndIsReq.getPdServiceId()),
                        new TermsQueryFilter("pdServiceVersions", pdServiceAndIsReq.getPdServiceVersions())
                );

        scopeDto.setEsQuery(esQuery);

        return this.findRecentHits(scopeDto).toDocs();
    }
}
