package com.arms.api.dashboard.service;

import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.issue.almapi.repository.지라이슈_저장소;
import com.arms.api.util.model.dto.PdServiceAndIsReqDTO;
import com.arms.api.util.model.dto.request.지라이슈_기본_검색_집계_하위_요청;
import com.arms.api.util.model.dto.response.제품버전목록;
import com.arms.api.util.model.dto.request.트리맵_검색_집계_하위_요청;
import com.arms.api.util.model.vo.TaskList;
import com.arms.api.util.model.vo.Worker;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.factory.creator.기본_쿼리_생성기;
import com.arms.egovframework.javaservice.esframework.factory.creator.중첩_집계_쿼리_생성기;
import com.arms.egovframework.javaservice.esframework.model.dto.기본_검색_요청;
import com.arms.egovframework.javaservice.esframework.model.vo.버킷_집계_결과_목록_합계;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service("지라이슈_대시보드_서비스")
@AllArgsConstructor
public class 지라이슈_대시보드_서비스_프로세스 implements 지라이슈_대시보드_서비스 {

    private 지라이슈_저장소 지라이슈_저장소;

    @Override
    public 버킷_집계_결과_목록_합계 제품_버전_요구사항_이슈_집계(Long pdServiceId, 지라이슈_기본_검색_집계_하위_요청 지라이슈_일반_집계_요청) {

        EsQuery esQuery
                = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", pdServiceId),
                        new TermsQueryFilter("pdServiceVersions", 지라이슈_일반_집계_요청.getPdServiceVersionLinks()),
                        new TermQueryMust("isReq", 지라이슈_일반_집계_요청.getIsReq())
                );

        return 지라이슈_저장소.aggregateRecentDocs(중첩_집계_쿼리_생성기.포괄(지라이슈_일반_집계_요청, esQuery).생성());
    }

    @Override
    public List<Worker> 작업자_별_요구사항_별_관여도(트리맵_검색_집계_하위_요청 트리맵_집계_요청) {
        Map<String, Worker> contributionMap = new HashMap<>();

        List<제품버전목록> 제품버전목록데이터 = 트리맵_집계_요청.get제품버전목록();


        //2024.11.21 이민규
        //재리팩토링 필요 start
        PdServiceAndIsReqDTO pdServiceAndIsReqDto = new PdServiceAndIsReqDTO();
        pdServiceAndIsReqDto.setIsReq(true);
        pdServiceAndIsReqDto.setPdServiceVersions(트리맵_집계_요청.getPdServiceVersionLinks());
        pdServiceAndIsReqDto.setPdServiceId(트리맵_집계_요청.getPdServiceLink());
        //재리팩토링 필요 end
       
        List<AlmIssueEntity> requirementIssues = this.findIssue(pdServiceAndIsReqDto);

        // 요구사항의 키를 모두 추출
        List<String> allReqKeys = requirementIssues.stream().map(AlmIssueEntity::getKey).collect(Collectors.toList());

        // 모든 하위 태스크를 한 번에 로드
        List<AlmIssueEntity> allSubTasks = this.하위이슈_조회(allReqKeys);

        // 하위 태스크를 부모 키로 그룹화
        Map<String, List<AlmIssueEntity>> subTasksByParent = allSubTasks.stream()
                .filter(subtask -> subtask.getAssignee() != null)
                .collect(Collectors.groupingBy(AlmIssueEntity::getParentReqKey));

        requirementIssues.stream().forEach(reqIssue -> {
            String key = reqIssue.getKey();
            String summary = reqIssue.getSummary() == null ? "해당 요구사항은 지라서버에서 조회가 되지 않는 상태입니다." : reqIssue.getSummary();
            List<Long> pdServiceVersions = reqIssue.getPdServiceVersions();
            String versionNames =  pdServiceVersions.stream()
                    .map(versionId -> 제품버전목록데이터.stream()
                            .filter(p -> p.getC_id().equals(versionId.toString()))
                            .findFirst()
                            .map(제품버전목록::getC_title)
                            .orElse(null)
                    )
                    .filter(Objects::nonNull)
                    .collect(Collectors.joining(", "));

            Optional.ofNullable(subTasksByParent.get(key)).orElse(Collections.emptyList()).stream().forEach(subtask -> {
                String assigneeId = subtask.getAssignee().getAccountId();
                String displayName = subtask.getAssignee().getDisplayName();

                Worker worker = contributionMap.computeIfAbsent(assigneeId, id -> {
                    Map<String, Integer> dataMap = new HashMap<>();
                    dataMap.put("totalInvolvedCount", 0);
                    return new Worker(assigneeId, displayName, dataMap, new ArrayList<>());
                });

                TaskList taskList = worker.getChildren().stream()
                        .filter(task -> task.getId().equals(key))
                        .findFirst()
                        .orElseGet(() -> {
                            Map<String, Integer> dataList = new HashMap<>();
                            dataList.put("involvedCount", 0);
                            TaskList newTask = new TaskList(key, "[ " + versionNames + " ] - " + summary, dataList);
                            worker.getChildren().add(newTask);
                            return newTask;
                        });

                taskList.getData().put("involvedCount", taskList.getData().get("involvedCount") + 1);
                worker.getData().put("totalInvolvedCount", worker.getData().get("totalInvolvedCount") + 1);
            });
        });

        return contributionMap.values().stream()
                .sorted((w1, w2) -> w2.getData().get("totalInvolvedCount").compareTo(w1.getData().get("totalInvolvedCount")))
                .limit(트리맵_집계_요청.get크기() > 0 ? 트리맵_집계_요청.get크기() : Long.MAX_VALUE)
                .collect(Collectors.toList());

    }


    private List<AlmIssueEntity> findIssue(PdServiceAndIsReqDTO pdServiceAndIsReqDto) {
        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                new TermsQueryFilter("isReq", pdServiceAndIsReqDto.getIsReq()),
                new TermsQueryFilter("pdServiceId", pdServiceAndIsReqDto.getPdServiceId()),
                new TermsQueryFilter("pdServiceVersions", pdServiceAndIsReqDto.getPdServiceVersions())
            );
        return 지라이슈_저장소.findRecentHits(기본_쿼리_생성기.기본검색(esQuery).생성()).toDocs();
    }


    private List<AlmIssueEntity> 하위이슈_조회(List<String> parentReqKeys) {
        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                new TermsQueryFilter("parentReqKey", parentReqKeys)
            );
        return 지라이슈_저장소.findRecentHits(기본_쿼리_생성기.기본검색(esQuery).생성()).toDocs();
    }



}
