package com.arms.api.issue.priority.service;

import com.arms.api.issue.priority.model.IssuePriorityDTO;
import com.arms.api.issue.priority.strategy.*;
import com.arms.api.serverinfo.model.enums.서버유형_정보;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.serverinfo.service.서버정보_서비스;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class IssuePriorityService {

    private 서버정보_서비스 서버정보_서비스;
    private CloudJiraIssuePriorityStrategy cloudJiraIssuePriorityStrategy;
    private OnpremiseJiraIssuePriorityStrategy onpremiseJiraIssuePriorityStrategy;
    private OnpremiseRedmineIssuePriorityStrategy onpremiseRedmineIssuePriorityStrategy;

    @Autowired
    public IssuePriorityService(서버정보_서비스 서버정보_서비스,
                                CloudJiraIssuePriorityStrategy cloudJiraIssuePriorityStrategy,
                                OnpremiseJiraIssuePriorityStrategy onpremiseJiraIssuePriorityStrategy,
                                OnpremiseRedmineIssuePriorityStrategy onpremiseRedmineIssuePriorityStrategy) {

        this.서버정보_서비스 = 서버정보_서비스;
        this.cloudJiraIssuePriorityStrategy = cloudJiraIssuePriorityStrategy;
        this.onpremiseJiraIssuePriorityStrategy = onpremiseJiraIssuePriorityStrategy;
        this.onpremiseRedmineIssuePriorityStrategy = onpremiseRedmineIssuePriorityStrategy;
    }

    private IssuePriorityStrategyImpl settingIssuePriorityStrategy(서버정보_데이터 서버정보) {

        if (서버정보 == null || 서버정보.getType().isEmpty()) {
            log.error("settingIssuePriorityStrategy Error: 서버정보_유형 {}" , ErrorCode.서버유형_정보오류.getErrorMsg());
            throw new IllegalArgumentException("settingIssuePriorityStrategy Error: 서버정보_유형 " + ErrorCode.서버유형_정보오류.getErrorMsg());
        }

        서버유형_정보 서버_유형 = 서버유형_정보.valueOf(서버정보.getType());

        IssuePriorityStrategy issuePriorityStrategy;
        if (서버_유형 == 서버유형_정보.클라우드) {
            issuePriorityStrategy = this.cloudJiraIssuePriorityStrategy;
        }
        else if (서버_유형 == 서버유형_정보.온프레미스) {
            issuePriorityStrategy = this.onpremiseJiraIssuePriorityStrategy;
        }
        else if (서버_유형 == 서버유형_정보.레드마인_온프레미스) {
            issuePriorityStrategy = this.onpremiseRedmineIssuePriorityStrategy;
        }        
        else {
            throw new IllegalArgumentException("settingIssuePriorityStrategy Error: 허용하지 않는 서버정보_유형입니다. :: "+ 서버_유형+ " :: " + ErrorCode.서버유형_정보오류.getErrorMsg());
        }

        IssuePriorityStrategyImpl issuePriorityStrategyImpl = new IssuePriorityStrategyImpl();
        issuePriorityStrategyImpl.registerIssuePriorityStrategy(issuePriorityStrategy);

        return issuePriorityStrategyImpl;
    }

    public List<IssuePriorityDTO> getIssuePriorityList(Long connectId) {

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(connectId);

        IssuePriorityStrategyImpl issuePriorityStrategy = settingIssuePriorityStrategy(서버정보);

        List<IssuePriorityDTO> issuePriorityList = issuePriorityStrategy.getIssuePriorityList(서버정보);
        return issuePriorityList;
    }

}
