package com.arms.api.issue.status.controller;

import com.arms.api.issue.status.service.IssueStatusService;
import com.arms.api.issue.status.vo.CloudJiraByIssueTypeIssueStatusVO;
import com.arms.api.issue.status.vo.IssueStatusVO;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Slf4j
@RestController
@RequestMapping("/{connectId}/jira/issuestatus")
public class IssueStatusController {

    private final IssueStatusService issueStatusService;

    @Autowired
    public IssueStatusController(IssueStatusService issueStatusService) {
        this.issueStatusService = issueStatusService;
    }

    @ResponseBody
    @RequestMapping(
            value = {"/list"},
            method = {RequestMethod.GET}
    )
    public List<IssueStatusVO> getIssueStatusList(@PathVariable("connectId") Long connectId) {

        if (connectId == null) {
            log.error("getIssueStatusList Error: connectId {}", ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("getIssueStatusList Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        return issueStatusService.getIssueStatusList(connectId);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/project/{projectId}"},
            method = {RequestMethod.GET}
    )
    public List<CloudJiraByIssueTypeIssueStatusVO> getIssueStatusListByProject(@PathVariable("connectId") Long connectId,
                                                                               @PathVariable("projectId") String projectId) {

        if (connectId == null) {
            log.error("getIssueStatusListByProject Error: connectId {}", ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("getIssueStatusListByProject Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (StringUtils.isEmpty(projectId)) {
            log.error("getIssueStatusListByProject Error projectId {}", ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("getIssueStatusListByProject Error projectId " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        return issueStatusService.getIssueStatusListByProject(connectId, projectId);
    }
}
