package com.arms.api.issue.type.controller;

import com.arms.api.issue.type.vo.IssueTypeVO;
import com.arms.api.issue.type.service.IssueTypeService;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Slf4j
@RestController
@RequestMapping("/{connectId}/jira/issuetype")
@AllArgsConstructor
public class IssueTypeController {

    private final IssueTypeService issueTypeService;

    @ResponseBody
    @RequestMapping(
            value = {"/list"},
            method = {RequestMethod.GET}
    )
    public List<IssueTypeVO> getIssueTypeList(@PathVariable("connectId") Long connectId) {

        if (connectId == null) {
            log.error("getIssueTypeList Error: connectId {}", ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("getIssueTypeList Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        return issueTypeService.getIssueTypeList(connectId);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/project/{projectId}"},
            method = {RequestMethod.GET}
    )
    public List<IssueTypeVO> getIssueTypeListByProject(@PathVariable("connectId") Long connectId,
                                                       @PathVariable("projectId") String projectId) {

        if (connectId == null) {
            log.error("getIssueTypeListByProject Error: connectId {}", ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("getIssueTypeListByProject Error: connectId " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (StringUtils.isEmpty(projectId)) {
            log.error("getIssueTypeListByProject Error projectId {}", ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("getIssueTypeListByProject Error projectId " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        return issueTypeService.getIssueTypeListByProject(connectId, projectId);
    }
}
