package com.arms.api.issue.type.strategy;

import com.arms.api.issue.type.vo.IssueTypeVO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.지라유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import com.atlassian.jira.rest.client.api.JiraRestClient;
import com.atlassian.jira.rest.client.api.domain.IssueType;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

@Component
@AllArgsConstructor
public class OnpremiseJiraIssueTypeStrategy implements IssueTypeStrategy {

    private final 지라유틸 지라유틸;

    @Override
    public List<IssueTypeVO> getIssueTypeList(서버정보_데이터 서버정보) {

        JiraRestClient restClient = 지라유틸.온프레미스_통신기_생성(서버정보.getUri(),
                서버정보.getUserId(),
                서버정보.getPasswordOrToken());

        try {
            Iterable<IssueType> onpremiseJiraIssueTypeList = restClient.getMetadataClient().getIssueTypes().get();

            return StreamSupport.stream(onpremiseJiraIssueTypeList.spliterator(), false)
                    .map(this::convertIssueTypeVO)
                    .collect(Collectors.toList());
        }
        catch (InterruptedException e) {
            String errorMessage =
                    ErrorLogUtil.exceptionLoggingAndReturn(e,
                            this.getClass().getName(),
                            String.format("%s[%s] :: getIssueTypeList InterruptedException Error. Thread was interrupted",
                                    서버정보.getType(), 서버정보.getUri())
                    );
            Thread.currentThread().interrupt();
            throw new IllegalStateException(errorMessage);
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssueTypeList Error", 서버정보.getType(), 서버정보.getUri()));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    @Override
    public List<IssueTypeVO> getIssueTypeListByProject(서버정보_데이터 서버정보, String projectId) { return null; }

    private IssueTypeVO convertIssueTypeVO(IssueType issueType) {
        return IssueTypeVO.builder()
                .self(issueType.getSelf().toString())
                .id(issueType.getId().toString())
                .description(issueType.getDescription())
                .name(issueType.getName())
                .subtask(issueType.isSubtask()).build();
    }
}
