package com.arms.egovframework.javaservice.esframework.model.vo.v2;

import java.util.List;
import java.util.stream.Collectors;

import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.search.aggregations.bucket.MultiBucketsAggregation.Bucket;

import lombok.Getter;

@Slf4j
public class DocumentBucket {

    @Getter
    private String groupKey;

    @Getter
    private String docFieldName;

    @Getter
    private long docCount;

    private List<DocumentBucket> subDocuments;

    @Setter
    private DocumentBucket parentDocumentBucket;

    private DocumentBucket() {
    }

    List<DocumentBucket> getDeepestSubDocument() {

        if (subDocuments == null || subDocuments.isEmpty()) {
            return List.of(this);
        }

        subDocuments
            .forEach(bucket -> {
                bucket.setParentDocumentBucket(this);
            });

        return subDocuments.stream()
                .flatMap(bucket -> {
                    List<DocumentBucket> deepestSubList = bucket.getDeepestSubDocument();
                    return deepestSubList.stream();
                })
                .collect(Collectors.toList());
    }


    public String getDocFieldName(String docAliasName) {

        if (parentDocumentBucket == null&&docAliasName.equals(groupKey)) {
            return docFieldName;
        }

        if (parentDocumentBucket == null) {
            return "";
        }

        if(docAliasName.equals(groupKey)) {
            return docFieldName;
        }

        return parentDocumentBucket.getDocFieldName(docAliasName);
    }

    public Long getDocFieldNameCount(String docAliasName) {

        if (parentDocumentBucket == null&&docAliasName.equals(groupKey)) {
            return docCount;
        }

        if (parentDocumentBucket == null) {
            return 0L;
        }

        if(docAliasName.equals(groupKey)) {
            return docCount;
        }

        return parentDocumentBucket.getDocFieldNameCount(docAliasName);
    }

    DocumentBucket(String groupKey,Bucket bucket) {

        this.groupKey = groupKey;

        this.docFieldName = bucket.getKeyAsString();

        this.docCount = bucket.getDocCount();

        this.subDocuments = new DocumentAggregations(bucket.getAggregations().asMap()).docBuckets();

    }

}
