package com.arms.egovframework.javaservice.esframework.repository;

import com.arms.egovframework.javaservice.esframework.repository.공통저장소;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Component
@AllArgsConstructor
class FindRepository {
    private final List<공통저장소<?, ?>> repositories;

    public <T,ID extends Serializable> 공통저장소<T, ID> findRepositoryByClass(Class<T> clazz) {

        return repositories.stream()
                .filter(repo -> {
                    Type[] genericInterfaces = repo.getClass().getGenericInterfaces();
                    List<Type> collect = Arrays.stream(genericInterfaces)
                            .flatMap(type -> {
                                Type[] generics = ((Class<?>)type).getGenericInterfaces();
                                return Arrays.stream(generics)
                                        .filter(generic -> generic instanceof ParameterizedType)
                                        .filter(generic -> ((ParameterizedType)generic).getActualTypeArguments()[0].equals(clazz));
                            }).collect(Collectors.toList());
                    return !collect.isEmpty();
                })
                .map(repo-> (공통저장소<T, ID>)repo)
                .findFirst()
                .orElseThrow(()->new IllegalArgumentException("타입이 존재 하지 않습니다."));
    }
}
